﻿local NekoLoot = LibStub("AceAddon-3.0"):NewAddon("NekoLoot", "AceEvent-3.0", "AceConsole-3.0")
local AceConfReg = LibStub("AceConfigRegistry-3.0")
local LBF = LibStub("LibButtonFacade", true)
local L = LibStub("AceLocale-3.0"):GetLocale("NekoLoot")
local media = LibStub("LibSharedMedia-3.0")
local db
local skins

local buttonFrames = {}
local buttons = {}

local type, ipairs, unpack = type, ipairs, unpack
local FreeSlots = GetContainerNumFreeSlots
local toolTip = self.tooltip

local defaults = {
	profile = {
		general = {
			descText = true,
			descColor = { 1, 0.8, 0},
			textWidth = 155,

			descHeight = 10,
			qualityHeight = 10,
			textHeight = 10,
			bindHeight = 10,

			expand = true,
			highlight = 1,
		},

		frame = {
			scale = 1.0,
			alpha = 1.0,
			strata = "DIALOG",
			
			lock = true,
			
			parent = UIParent,
			anchorPoint = "CENTER",
			parentAnchor = "CENTER",
			xOffSet = 0,
			yOffSet = 0,
			width = 200,
			
			backgroundColor = { 0, 0, 0, 0.7 },
			borderColor = { 0.5, 0.5, 0.5, 1 },
		},
		
		loot = {
			height = 20,
			backgroundColor = { 0, 0, 0, 0.9 },
			buttonBorderColor = { 0.7, 0.7, 0.7, 1 },

			highlightIconColor = true,
			highlightButton = true,
			highlightFrame = false,

			-- no option yet
			frameBorderColor = { 0.5, 0.5, 0.5, 1 },
		},

		lootMode = {
			override = false,
			fast = false,
			extraFast = false,
		},
	},
}

function NekoLoot:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("NekoLootDB", defaults, true)
	db = self.db.profile
	
	self:SetupOptions()
	
	-- create tooltip to scan for item binding
	self:CreateTooltip()

	self.lootFrame = nil
	self:CreateLootFrame()
end

-- Add addon handler to events and disable blizzards defaults
function NekoLoot:OnEnable()
	LootFrame:UnregisterEvent("LOOT_OPENED")
	LootFrame:UnregisterEvent("LOOT_SLOT_CLEARED")
	LootFrame:UnregisterEvent("LOOT_CLOSED")

	-- Register loot mode on loot event
	if db.lootMode.fast then
		self:ChangeLooting("fast", true)
	elseif db.lootMode.extraFast then
		self:ChangeLooting("extraFast", true)
	else
		self:ChangeLooting()
	end
	
	self:RegisterEvent("LOOT_SLOT_CLEARED", "LootSlotClear")
	-- Fired when loot is removed from a corpse. 

	self:RegisterEvent("LOOT_CLOSED", "LootClosed")
	-- Fired when a player ceases looting a corpse. Note that this will fire before the last CHAT_MSG_LOOT event for that loot. 
	

	-- usefull events, may be support some times --
	-- Inventory is Full???
	
	-- UPDATE_BINDINGS
	-- AutoLootModified changed???
	
	-- CANCEL_LOOT_ROLL
	-- Fired when a player cancels a roll on an item 
	
	-- CONFIRM_DISENCHANT_ROLL
	-- Fires when you try to roll "disenchant" for and item which Binds on Pickup. 
	
	-- CONFIRM_LOOT_ROLL
	-- Fires when you try to roll "need" or "greed" for and item which Binds on Pickup. 
	
	-- LOOT_BIND_CONFIRM
	-- Fired when the player attempts to take 'bind-on-pickup' loot
	
	-- START_LOOT_ROLL
	-- Fired when a group loot item is being rolled on.
	
	-- "UPDATE_MASTER_LOOT_LIST"
	-- ???

	-- register for setting changed
	self.db.RegisterCallback(self, "OnProfileChanged", "ApplySettings")
	self.db.RegisterCallback(self, "OnProfileCopied", "ApplySettings")
	self.db.RegisterCallback(self, "OnProfileReset", "ApplySettings")

	-- TODO use shared media!
	media.RegisterCallback(self, "LibSharedMedia_Registered", "ApplySettings")
	media.RegisterCallback(self, "LibSharedMedia_SetGlobal", "ApplySettings")

	-- support for configuration mode
	CONFIGMODE_CALLBACKS = CONFIGMODE_CALLBACKS or {}
	CONFIGMODE_CALLBACKS["NekoLoot"] = function(action)
		if action == "ON" then
			self:Unlock(false)
		elseif action == "OFF" then
			self:Lock()
		end
	end
	
	self:ApplySettings()
end

-- Remove anything done by this addon and add blizzards defaults
function NekoLoot:OnDisable()
	self:UnregisterAllEvents()

	LootFrame:RegisterEvent("LOOT_OPENED")
	LootFrame:RegisterEvent("LOOT_SLOT_CLEARED")
	LootFrame:RegisterEvent("LOOT_CLOSED")
end

-- Get Number of items of loot table and fill empty loot token in loot frame
-- if not enough loot token in loot frame add someone
function NekoLoot:OnUpdate()
end

-- apply settings
function NekoLoot:ApplySettings()
	db = self.db.profile

	self.lootFrame:SetFrameStrata(db.frame.strata)
	self.lootFrame:SetScale(db.frame.scale)
	self.lootFrame:SetAlpha(db.frame.alpha)

	self.lootFrame:SetMovable(not db.frame.lock)
	self.lootFrame:EnableMouse(not db.frame.lock)
	
	-- Hintergrund 	setzen und Skin auf LootFrame anwenden
	self:BackdropFrame(self.lootFrame, db.frame.backgroundColor, db.frame.borderColor)
	self:Skin(NekoLootFrame, db.frame.backgroundColor, db.frame.borderColor)

	-- Ausrichtung
   	self.lootFrame:ClearAllPoints()
	self.lootFrame:SetPoint(
		db.frame.anchorPoint,
		db.frame.parent,
		db.frame.parentAnchor,
		db.frame.xOffSet,
		db.frame.yOffSet
	)

	self:Update()
end

function NekoLoot:ToggleLock(showUI)
	local func = not db.frame.lock and "Lock" or "Unlock"
	self[func](self, showUI)
end

local function ConfigModeDialogHandler()
	if NekoLoot.dialog:IsShown() then
		NekoLoot.dialog:Hide()
		NekoLoot:Lock()		
		AceConfReg:NotifyChange("NekoLoot")
	end
end

-- make window moveable
function NekoLoot:Unlock(showUI)
	db.frame.lock = false

	self.lootFrame:Show()
	self.lootFrame:EnableMouse(true)
	self.lootFrame:SetMovable(true)
	self.lootFrame:SetAlpha(1)

	if showUI then
		NekoLoot:CreateDialog(
			"NekoLootUnlockDialog",
			L["UnlockDialog/AllUnlocked"],
			L["UnlockDialog/Lock"],
			"BOTTOMRIGHT",
			ConfigModeDialogHandler)
	end
end

-- lock window
function NekoLoot:Lock()
	db.frame.lock = true
	
	if GetNumLootItems() < 1 then
		self.lootFrame:Hide()
	end
	
	self.lootFrame:EnableMouse(false)
	
	if self.unlock_dialog then
		self.unlock_dialog:Hide()
	end
end

function NekoLoot:LootSlotClear(info, index)
	self:Update()
end

function NekoLoot:FastLootSlotClear(info, index)
	local prev, next, numLootItems

	-- Finde Button
	for frame_index, frame in ipairs(buttonFrames) do
		if frame.button.slot == index then
			index = frame_index
			break
		end
	end
	
	-- Vorgänger und Nachfolger suchen und zusammenfügen
	prev = buttonFrames[index - 1]
	next = buttonFrames[index + 1]
	
	-- button verstecken
	buttonFrames[index]:Hide()
	
	if prev ~= nil and next ~= nil then
		next:SetPoint("TOPLEFT", prev, "BOTTOMLEFT", 0, -2)
	elseif next ~= nil then
		next:SetPoint("TOPLEFT", self.lootFrame, "TOPLEFT", 10, -10)
	end
	numLootItems = GetNumLootItems() - 1

	self.lootFrame:SetHeight(20 + numLootItems * (NekoLootButtonFrame1:GetHeight() + 2))
end

function NekoLoot:ClearButtonFrame(index)
	if index == nil then return nil end
	local button = buttonFrames[index].button
	
	button.slot = nil
	button.text = nil
	button.desc = nil
	button.bind = nil
	SetItemButtonCount(button, 0)
	button:Hide()
	buttonFrames[index]:Hide()
end

function NekoLoot:OnChanged()

end

function NekoLoot:OnOpenMaster()

end

function NekoLoot:OnUpdateMaster()

end

function NekoLoot:OnHide()
	-- if config mode keep frame visible
	if self.unlock_dialog and self.unlock_dialog:IsVisible() then return end

	NekoLootFrame:Hide()
end

function NekoLoot:OnShow()
	NekoLootFrame:Show()
end

-- set addiotonal information of an item
-- lootLink			: link of the item
-- bind				: text obj where binding should be set
-- desc				: text obj where description of the item should be set
function NekoLoot:SetSlotInfo(lootLink, bind, desc)
	local _, _, rarity, _, _, itemType, subType, _, equipLoc = GetItemInfo(lootLink)
	local bindText, r, g, b = self:GetItemBinding(lootLink)
	
	-- set item binding text
	if bindText ~= nil then
		bind:SetText(string.sub(bindText, 1, 5))
		bind:SetVertexColor(r, g, b)
	end

	if itemType == ENCHSLOT_WEAPON then
		equipLoc = ENCHSLOT_WEAPON
	else
		equipLoc = _G[equipLoc]
	end

	if subType == "Junk" then 
		subType = (rarity > 0) and MISCELLANEOUS or subType
	end
	desc:SetText(((type(equipLoc) == "string" and equipLoc ~= "") and equipLoc..", " or "") .. ((subType == subType) and subType or subType.." "..itemType))
end

-- get item binding and the corresponding text color 
-- itemlink			: link of the item to scan
function NekoLoot:GetItemBinding(itemLink)
	if not itemLink then return nil end
	
	local numLines, text, textLine

	toolTip:SetOwner(UIParent, "ANCHOR_NONE")
	toolTip:SetHyperlink(itemLink)
	numLines = NekoLootTT:NumLines()
	
	-- search only in the first six lines
	numLines = math.min( numLines, 6)
	
	-- search only for BoP, BoE and BtA
	for index = 1, numLines do
		textLine = _G["NekoLootTTTextLeft"..index]
		text = textLine:GetText()

		if text == ITEM_ACCOUNTBOUND or text == ITEM_BIND_TO_ACCOUNT then
			toolTip:Hide()
			return L["BtAcc"], .9, .8, .5
		elseif text == ITEM_BIND_ON_EQUIP then
			toolTip:Hide()
			return L["BoE"], .3, 1, .3
		elseif text == ITEM_BIND_ON_PICKUP then
			toolTip:Hide()
			return L["BoP"], 1, .0, .1
		elseif text == ITEM_BIND_ON_USE then
			toolTip:Hide()
			return L["BoU"], .3, .5, 1
		end
	end
	toolTip:Hide()
	return "", 1, 1, 1
end

function NekoLoot:UpdateWidths(numFrames, maxWidth)
	local frameWidth = db.frame.width
	local buttonWidth, frameBorder = -(maxWidth + 16), nil

	-- set dynamic frame width
	if db.general.expand then
		frameWidth = (buttonFrames[1].button:GetWidth() + maxWidth + 21)
	end

	-- resize anything
	for index = 1, numFrames do
		curFrame = buttonFrames[index]
		curFrame:SetWidth(frameWidth)
		curFrame.button:SetHitRectInsets(0, buttonWidth, 0, -1)
		
		frameBorder = _G[curFrame:GetName().."QualBorder"]
		frameBorder:SetHeight(curFrame:GetHeight() * 1.62)
		frameBorder:SetWidth(frameWidth * 1.72)
		frameBorder:SetPoint("CENTER", curFrame, "CENTER", 5, 1)
	end
	self.lootFrame:SetWidth(frameWidth + 24)
end

function NekoLoot:Update()
	local curButtonIndex, maxWidth, textWidth = 0, 0, 0
	local curFrame, curButton, text, desc, quality, bind, numLootItems
	local lootIcon, lootName, lootQuantity, rarity
	local r, g, b, hex, test
	
	numLootItems = GetNumLootItems()
	while #buttonFrames < numLootItems do
		self:AddLootToken(#buttonFrames + 1)
	end
	
	-- Wenn dynamische Breite 700, sonst Breite aus Options
	textWidth = (db.general.expand and 700) or db.general.textWidth
	
	for index = 1, numLootItems do
		lootIcon, lootName, lootQuantity, rarity, _ = GetLootSlotInfo(index);
		if( lootIcon ~= nil) then
			curButtonIndex = curButtonIndex + 1
			
			curFrame = buttonFrames[curButtonIndex]
			curButton = curFrame.button

			curButton:SetID(index)
			curButton.slot = index
			r, g, b, hex = GetItemQualityColor(rarity)

			SetItemButtonTexture(curButton, lootIcon)
			
			text = _G["NekoLootButton"..curButtonIndex.."Text"]
			desc = _G["NekoLootButton"..curButtonIndex.."Description"]
			bind = _G["NekoLootButton"..curButtonIndex.."Bind"]
			--quality = _G["NekoLootButton"..curButtonIndex.."Quality"]
			desc:SetVertexColor(unpack(db.general.descColor))
			text:SetVertexColor(r, g, b)

			desc:SetWidth(textWidth)
			text:SetWidth(textWidth)
			SetItemButtonCount(curButton, lootQuantity)

			if LootSlotIsCoin(index) or LootSlotIsCurrency(index) then
				-- Setze Goldbetrag
				text:SetText(lootName:gsub('\n', ', '))
				text:SetPoint("TOPLEFT", curButton, "TOPLEFT", 42, 2)
				text:SetHeight(NekoLootButton1:GetHeight() + 1)
				desc:SetText("")
				bind:SetText("")
				--quality:SetText("")
				curButton.border:Hide()
				curFrame.border:Hide()
			else
				-- Setze Gegenstandstext
				text:SetText(('%s%s|r'):format(hex, lootName))
				if db.general.descText then
					text:SetPoint("TOPLEFT", curButton, "TOPLEFT", 42, -10)
					-- Setze Gegenstandsinformationen in die Felder Bind und Description
					self:SetSlotInfo(GetLootSlotLink(index), bind, desc)
					desc:SetPoint("TOPLEFT", curButton, "TOPLEFT", 45, -20)
					-- Wenn Gegenstandsinformationen gesetzt, verkleiner Gegenstandstext
					text:SetHeight(10)
				else
					text:SetPoint("TOPLEFT", curButton, "TOPLEFT", 42, -12)
				end
				
				-- Symbol und Frame einfärben
				if rarity >= db.general.highlight then
					-- Symbol nach Qualität
					if db.loot.highlightIcon then
						curButton.border:SetVertexColor(r, g, b)
						curButton.border:Show()
					else
						curButton.border:Hide()
					end
					-- ItemFrame nach Qualität
					if db.loot.highlightFrame then
						curFrame.border:SetVertexColor(r, g, b)
						curFrame.border:Show()
					else
						curFrame.border:Hide()
					end
				else
					curButton.border:Hide()
					curFrame.border:Hide()
				end
			end

			maxWidth = max(maxWidth, text:GetStringWidth(), desc:GetStringWidth())
			
			-- ItemButton nach Qualität
			if db.loot.highlightButton then
				-- border color based on rarity
				curFrame:SetBackdropBorderColor(r, g, b, 1)
				curButton.wrapper:SetBackdropBorderColor(r, g, b, 1)
			else
				curFrame:SetBackdropBorderColor(unpack(db.loot.buttonBorderColor))
				curButton.wrapper:SetBackdropBorderColor(unpack(db.loot.buttonBorderColor))
			end

			curButton:Show()
			curFrame:Show()
		end
	end
	for index = curButtonIndex + 1, #buttonFrames do
		buttonFrames[index]:Hide()
		buttonFrames[index].button:Hide()
	end
	
	-- Höhe berechnen und setzen
	self.lootFrame:SetHeight(20 + (curButtonIndex * (NekoLootButtonFrame1:GetHeight() + 2)))

	self:UpdateWidths(curButtonIndex, maxWidth)
	if numLootItems > 0 then self.lootFrame:Show() else self.lootFrame:Hide() end
end

-- Add to lootframe a single empty loot token
function NekoLoot:AddLootToken(index)
	local frame = CreateFrame("Frame", "NekoLootButtonFrame"..index, self.lootFrame)
	local fBorder
	local buttonName = "NekoLootButton"..index
	local button = CreateFrame("Button", buttonName, frame, "LootButtonTemplate")
	local bBorder

	local text = _G[buttonName.."Text"]
	local desc = button:CreateFontString(buttonName.."Description")
	--local quality = button:CreateFontString(buttonName.."Quality")
	local bind = button:CreateFontString(buttonName.."Bind")
	
	local font = { STANDARD_TEXT_FONT, 10, "" }
	local bindFont = { STANDARD_TEXT_FONT, 9, "OUTLINE" }

	text:SetDrawLayer("OVERLAY")
	desc:SetDrawLayer("OVERLAY")
	--quality:SetDrawLayer("OVERLAY")
	bind:SetDrawLayer("OVERLAY")

	desc:SetFont(unpack(font))
	--quality:SetFont(unpack(font))
	bind:SetFont(unpack(bindFont))

	desc:SetJustifyH("LEFT")
	--quality:SetJustifyH("LEFT")
	bind:SetJustifyH("LEFT")

	desc:SetHeight(db.general.descHeight)	
	--quality:SetHeight(db.general.qualityHeight)
	text:SetHeight(db.general.textHeight)
	bind:SetHeight(db.general.bindHeight)

	--quality:SetWidth(db.general.textWidth)

	--quality:SetPoint("TOPLEFT", button, "TOPLEFT", 45, -3)
	bind:SetPoint("BOTTOMLEFT",  button, "BOTTOMLEFT", 3, 3)
	
	button:SetHitRectInsets(0, -(db.frame.width - 40), 0, -1)

	-- End Template
	
	bBorder = self:QualityBorder(button)
	fBorder = self:QualityBorder(frame)
	button.wrapper = self:ItemButtonWrapper(button, 6, 6)

	fBorder:SetHeight(fBorder:GetHeight() -3)
	fBorder:SetPoint("CENTER", frame, "CENTER", 4, .5)
	fBorder:SetAlpha(0.3)
	frame:SetWidth(db.frame.width)
	frame:SetHeight(button:GetHeight() + 1)
	button:ClearAllPoints()
	frame:ClearAllPoints()
	if (index > 1) then 
		frame:SetPoint("TOPLEFT", buttonFrames[index-1], "BOTTOMLEFT", 0, -2)
	else
		frame:SetPoint("TOPLEFT", self.lootFrame, "TOPLEFT", 10, -10)
	end
	button:SetPoint("LEFT", frame, "LEFT")
	button:RegisterForDrag("LeftButton")
	button:SetScript("OnDragStart", StartMoving)
	button:SetScript("OnDragStop", StopMoving)
	button:SetScript("OnEnter", function(self) NekoLoot:OnEnter(self) end)
	button:SetScript("OnUpdate", function(self, elapsed) CursorOnUpdate(self) end)

	button.border = bBorder
	frame.button = button
	frame.border = fBorder
	buttonFrames[index] = frame
	self.lootFrame:SetHeight(self.lootFrame:GetHeight() + frame:GetHeight())

	--Skin
	self:Skin(frame, db.loot.backgroundColor, db.loot.buttonBorderColor)
	
	button.text = text
	button.desc = desc
	button.bind = bind
	--button.quality = quality
	button:DisableDrawLayer("ARTWORK")
	frame:Hide()
	button:Hide()
end

-- EventHandler für einen einzelnes LootItem
function NekoLoot:OnEnter(self)
	if LootSlotIsItem(self.slot) then
		GameTooltip:SetOwner(self, 'ANCHOR_TOPLEFT', 32, 0)
		GameTooltip:SetLootItem(self.slot)
		if IsShiftKeyDown() then
			GameTooltip_ShowCompareItem()
		end
		CursorUpdate(self)
	end
end

local function StartMoving() 
	if not db.frame.lock then
		NekoLootFrame:StartMoving() 
	end
end

local function StopMoving()
	if not db.frame.lock then 
		NekoLootFrame:StopMovingOrSizing()
		
		db.frame.xOffSet = NekoLootFrame:GetLeft()
		db.frame.yOffSet = NekoLootFrame:GetTop()
	end
end 

-- Creates the loot frame with ten empty loot token, that all are hidden
function NekoLoot:CreateLootFrame()
	-- Frame Eigenschaften
	self.lootFrame = CreateFrame("Frame", "NekoLootFrame", UIParent)
	self.lootFrame:SetFrameStrata(db.frame.strata)
	self.lootFrame:SetWidth(db.frame.width)
	self.lootFrame:SetHeight(db.loot.height)
	
	self.lootFrame:SetMovable(not db.frame.lock)
	self.lootFrame:EnableMouse(not db.frame.lock)
	
	-- Events und Handler
	self.lootFrame:RegisterForDrag("LeftButton")
	self.lootFrame:SetScript("OnShow", function(self) NekoLoot:OnShow() end)
	self.lootFrame:SetScript("OnHide", function(self) NekoLoot:OnHide() end)
	self.lootFrame:SetScript("OnDragStart", StartMoving)
	self.lootFrame:SetScript("OnDragStop", StopMoving)

	-- Hintergrund
	self:BackdropFrame(self.lootFrame, db.frame.backgroundColor, db.frame.borderColor)
   	self.lootFrame:ClearAllPoints()
	
	-- Skin auf LootFrame anwenden
	self:Skin(NekoLootFrame, db.frame.backgroundColor, db.frame.borderColor)

	self.lootFrame:SetScale(db.frame.scale)
	self.lootFrame:SetAlpha(db.frame.alpha)

	self:AddCloseButton(NekoLootFrame)

	-- Ausrichtung
	self.lootFrame:SetPoint(
		db.frame.anchorPoint,
		db.frame.parent,
		db.frame.parentAnchor,
		db.frame.xOffSet,
		db.frame.yOffSet
	)


	self:AddLootToken(1)
	self.lootFrame:Hide()
end

function NekoLoot:AddCloseButton(NekoLootFrame)
	local closeButton = CreateFrame("Button", "NLCloseButton", NekoLootFrame)

	closeButton:SetScript("OnClick", function(self) NekoLoot:LootClosed() end)
	closeButton:SetFrameLevel(8)
	closeButton:SetWidth(32)
	closeButton:SetHeight(32)
	closeButton:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
	closeButton:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down")
	closeButton:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	closeButton:ClearAllPoints()
	closeButton:SetPoint("TOPRIGHT", NekoLootFrame, "TOPRIGHT", 3, 3)
	closeButton:SetHitRectInsets(5, 5, 5, 5)
	closeButton:Show()
end

function NekoLoot:LootClosed()
	for index = 1, #buttonFrames do
		self:ClearButtonFrame(index)
	end
	-- if config mode keep frame visible
	if self.unlock_dialog and self.unlock_dialog:IsVisible() then
		return 
	end
	CloseLoot()
	self.lootFrame:Hide()
end